@@ -11,10 +11,24 @@ from galpy.util import bovy_conversion
 import limepy
 from limepy import limepy
 
+try:
+    from galpy.util import coords,conversion
+except:
+    import galpy.util.bovy_coords as coords
+    import galpy.util.bovy_conversion as conversion
+
+try:
+    import amuse.units.units as u
+except:
+		pass
+
+import time as time
+
 solar_motion=[-11.1,12.24,7.25] #Schönrich, R., Binney, J., Dehnen, W., 2010, MNRAS, 403, 1829
 solar_ro=8.275 #Gravity Collaboration, Abuter, R., Amorim, A., et al. 2020 ,A&A, 647, A59
 solar_vo=solar_ro*30.39-solar_motion[1]
 solar_zo=0.0208 #Bennett & Bovy 2019
+to=conversion.time_in_Gyr(ro=solar_ro,vo=solar_vo)
 
 def test_find_centre_of_density(tol=0.01):
 	cluster=ctools.load_cluster(ctype='limepy',gcname='NGC6101')
@@ -50,10 +64,6 @@ def test_find_centre_of_density_casertano(tol=0.01):
 	assert np.fabs(1.0-vyc/o.vy()) <= tol
 	assert np.fabs(1.0-vzc/o.vz()) <= tol
 
-	xc, yc, zc, vxc, vyc, vzc, rhos=ctools.find_centre_of_density(cluster,method='casertano',return_rhos=True)
-
-	assert len(rhos)==cluster.ntot
-
 
 def test_find_centre_of_mass(tol=0.01):
 	cluster=ctools.load_cluster(ctype='limepy',gcname='NGC6101')
@@ -120,6 +130,19 @@ def test_relaxation_time(tol=0.01):
 
 	assert np.fabs(1.0-trelax/cluster.relaxation_time()) <= tol
 
+	#Test calculation in Gyr
+	cluster.to_kpckms()
+	assert np.fabs(1.0-(trelax/1000.0)/cluster.relaxation_time()) <= tol
+
+	#Test calculation in galpy
+	cluster.to_galpy()
+	assert np.fabs(1.0-(trelax/1000.0/to)/cluster.relaxation_time()) <= tol
+
+	#Test calculation in amuse
+	cluster.to_amuse()
+
+	assert np.fabs(1.0-(trelax | u.Myr)/cluster.relaxation_time()) <= tol
+
 def test_projected_relaxation_time(tol=0.01):
 	cluster=ctools.StarCluster(units='pckms',origin='centre')
 
@@ -151,6 +174,11 @@ def test_projected_relaxation_time(tol=0.01):
 
 	assert np.fabs(1.0-trelax/cluster.relaxation_time(projected=True)) <= tol
 
+
+	#Test calculation in Gyr
+	cluster.to_kpckms()
+	assert np.fabs(1.0-(trelax/1000.0)/cluster.relaxation_time(projected=True)) <= tol
+
 def test_half_mass_relaxation_time(tol=0.01):
 	cluster=ctools.StarCluster(units='pckms',origin='centre')
 
@@ -188,7 +216,7 @@ def test_core_relaxation_time(tol=0.01):
 	#Setup cluster with rad=1, mean velocity=1, mass=ntot
 	ntot=1000
 	grav=4.302e-3
-	x=np.append(np.ones(int(ntot/2)),np.ones(int(ntot/2))*2.)
+	x=np.append(np.random.rand(int(ntot/2)),1.0+np.random.rand(int(ntot/2)))
 	y,z=np.zeros(ntot),np.zeros(ntot)
 
 	m=np.ones(ntot)
@@ -199,7 +227,8 @@ def test_core_relaxation_time(tol=0.01):
 	cluster.analyze()
 
 	coulomb=0.4
-	rc,rh=1.,1.
+	rh=1.
+	rc=cluster.rcore()
 	mbar=1.
 	vol=4.0*np.pi/3.0
 	rho=(ntot/2.)/vol
@@ -288,6 +317,82 @@ def test_energies(tol=0.01):
 	assert np.fabs(1.0-0.1*0.1*ptot/cluster.ptot) <= tol
 	np.testing.assert_array_equal(0.1*kin+0.1*0.1*pot,cluster.etot)
 
+def test_softened_energies(tol=0.01):
+
+	softening=0.1
+
+	cluster=ctools.StarCluster(units='nbody',origin='centre')
+
+	m=np.array([1.,1.])
+	x=np.array([-1.,1.])
+	vx=np.array([-2.,2.])
+
+	y,z=np.zeros(2)
+	vy,vz=np.zeros(2)
+
+	dx=np.array([2.,2.])
+
+	kin=0.5*vx**2.
+	pot=-1./np.sqrt(dx**2.+softening**2.)
+	etot=kin+pot
+	ektot=np.sum(kin)
+	ptot=np.sum(pot)/2.
+
+	cluster.add_stars(x,y,z,vx,vy,vz,m=m)
+	cluster.analyze()
+	cluster.energies(softening=softening)
+
+	assert np.fabs(1.0-ektot/cluster.ektot) <= tol
+	assert np.fabs(1.0-ptot/cluster.ptot) <= tol
+	np.testing.assert_array_equal(etot,cluster.etot)
+
+	cluster.energies(parallel=True,softening=softening)
+
+	assert np.fabs(1.0-ektot/cluster.ektot) <= tol
+	assert np.fabs(1.0-ptot/cluster.ptot) <= tol
+	np.testing.assert_array_equal(etot,cluster.etot)
+
+	kid,pid=cluster.energies(ids=cluster.id[0],softening=softening)
+	eid=kid+pid
+
+
+	assert(eid[0]==etot[0])
+
+	kid,pid=cluster.energies(ids=[cluster.id[0],cluster.id[1]],softening=softening)
+	eid=kid+pid
+
+	assert(eid[0]==etot[0])
+	assert(eid[1]==etot[1])
+
+	ids=np.ones(cluster.ntot,dtype=bool)
+
+	kid,pid=cluster.energies(ids=ids,softening=softening)
+	eid=kid+pid
+	assert(eid[0]==etot[0])
+	assert(eid[1]==etot[1])
+
+	ids[0]=False
+	kid,pid=cluster.energies(ids=ids,softening=softening)
+	eid=kid+pid
+	assert(eid[0]==etot[1])
+
+	cluster.z=x
+	cluster.vz=vx
+	cluster.analyze()
+	cluster.energies(projected=True,softening=softening)
+
+	assert np.fabs(1.0-ektot/cluster.ektot) <= tol
+	assert np.fabs(1.0-ptot/cluster.ptot) <= tol
+	np.testing.assert_array_equal(etot,cluster.etot)
+
+	cluster.m=np.array([0.1,0.1])
+	cluster.analyze()
+	cluster.energies(projected=True,specific=False,softening=softening)
+
+	assert np.fabs(1.0-0.1*ektot/cluster.ektot) <= tol
+	assert np.fabs(1.0-0.1*0.1*ptot/cluster.ptot) <= tol
+	np.testing.assert_array_equal(0.1*kin+0.1*0.1*pot,cluster.etot)
+
 def test_closest_star(tol=0.01):
 
 	x,y,z=np.linspace(0,4,5),np.zeros(5),np.zeros(5)
@@ -299,9 +404,15 @@ def test_closest_star(tol=0.01):
 	mindx=ctools.closest_star(cluster)
 	np.testing.assert_array_equal(mindx,np.ones(5))
 
+	dist,arg=ctools.closest_star(cluster,argument=True)
+	np.testing.assert_array_equal(mindx,dist)
+
 	cluster.z=cluster.x
 	mindx=ctools.closest_star(cluster,projected=True)
 	np.testing.assert_array_equal(mindx,np.ones(5))
+	
+	dist,arg=ctools.closest_star(cluster,argument=True,projected=True)
+	np.testing.assert_array_equal(mindx,dist)
 
 
 def test_rlagrange(tol=0.01):
